div.blogwrapper {
    font-size: 14px;
}
div.blogwrapper img {
    padding: 1em;
    /* the following 3 lines taken verbatim from bootstrap's .img-responsive */
    max-width: 100%;
    display: block;
    height: auto;
}
ul.invisible li {
    background: none repeat scroll 0 0 rgba(0, 0, 0, 0);
}
span.previousmonth {
    float: left;
}
span.nextmonth {
    float: right;
}
div.author_date div {
    color: #999999;
    float: right;
    padding-top: 3px;
}
div.author_date {
    border-top: thin ridge #ccc;
    font-size: 1.2em;
    font-style: italic;
}
div.microblog {
    clear: both;
}
div.microblog span.author {
    float: left;
    margin: 0 10px 5px 0;
}
div.microblog span.msgtxt {
    margin: 0 0 10px;
}
div.microblog span.meta {
    color: #999;
    display: block;
}
div.microblog span.meta a {
    color: #999;
}
